/*
 * Jukebox3D / esp Engine v2
 *
 * Copyright (C) 2005, 2007 Sylvain Pasquet (Sylvain.pasquet@gmail.com - pasquet.sylvain2@wanadoo.fr)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
/*************************************************
 *           interface_ogl.c    				 *
 *************************************************/
#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <gtk/gtk.h>
#include <gtk/gtkgl.h>
#include <string.h>
#include <GL/gl.h>
#include <GL/glu.h>
#include <gdk/gdkkeysyms.h>
#include <glade/glade-xml.h>

#include "../include/main.h"
#include "../include/model.h"
#include "../include/LibOgl.h"
#include "../include/LibImg.h"
#include "../include/interface_ogl.h"
#define TIMEOUT_INTERVAL 6 // Limit FPS
#define VIEW_SCALE_MAX 2.0
#define VIEW_SCALE_MIN 0.0
#define VIEW_INIT_AXIS_X 1.0
#define VIEW_INIT_AXIS_Y 0.0
#define VIEW_INIT_AXIS_Z 0.0
#define VIEW_INIT_ANGLE  20.0
#define DIG_2_RAD (G_PI / 180.0)
#define RAD_2_DIG (180.0 / G_PI)

void DrawSceneText(float myAlpha);
void SceneStyleAnim(float *fAngleObject,int iRot);
void myFX(int FX,GLenum mode,float *fValue,int *iStatus,float fStopFx);

/**Needed by main opengl scene**********************************/
static guint    timeout_id      = 0   ;
static gboolean animate         = TRUE;
/**Reflexion***/
static GLfloat reflexpwr = 0.6;
GtkWidget      *wOgl;
float points[45][45][3]  ; /* The Points On The Grid Of Our "Wave" */
/**Matrix View/Management********************/
static float    view_scale   = 1.0      ;
static float    view_quat[4] = { 0.0, 0.0, 0.0, 1.0 };
/**default size**/
int   SW=1024;
int   SH=768;
/**Main Camera localisation ************************************/
static GLfloat  view_rotx = 4.0, view_roty = -3.0, view_rotz =0.0 ;
static GLfloat  cam_rotx  = 4.0, cam_roty =-3.0, cam_rotz = 13.0;
/**Timer and FPS count***********************/
static GTimer   *timerFPS        = NULL     ;
static gint     frames           = 0        ;
/**Timer for text****************************/
gdouble  seconds       ;
/**Timer for FPS*****************************/
gdouble  fps           ;
/********************************************/
/**Flag system**/
int wiggle_count = 0;    /* Counter Used To Control How Fast Flag Waves */
GLfloat hold;            /* Temporarily Holds A Floating Point Value */
/**Flag system**/
int fx=0, fy=0;                   /* Loop Variables */
float f_x=0.0, f_y=0.0, f_xb=0.0, f_yb=0.0; /* Used To Break The Flag Into Tiny Quads */
float fXView=0.0  ;
float fYView=0.0  ;
/**Manage virtual archive mode**/
static int iArchInd=0;
void ReConfigureText(void)
{
	ConfigureText(wOgl);
}


gboolean
key_press_event (GtkWidget * widget, GdkEventKey * event, gpointer data)
{
	switch (event->keyval)
	{
		case GDK_Page_Up:
			if (OglScene->__iMode__==1) //If edit mode
			{
				cam_roty += 0.2;
			}
			if (OglScene->__iMode__==2) //If view mode
			{
				view_roty += 0.2;
			}
			break;

		case GDK_Page_Down:
			if (OglScene->__iMode__==1) //If edit mode
			{
				cam_roty -= 0.2;
			}
			if (OglScene->__iMode__==2) //If view mode
			{
				view_roty -= 0.2;
			}
			break;

		case GDK_Up:
			if (OglScene->__iMode__==1) //If edit mode
			{
				cam_rotz -= 0.2;
			}
			if (OglScene->__iMode__==2) //If view mode
			{
				view_rotz -= 0.2;
			}
			break;

		case GDK_Down:
			if (OglScene->__iMode__==1) //If edit mode
			{
				cam_rotz += 0.2;
			}
			if (OglScene->__iMode__==2) //If view mode
			{
				view_rotz += 0.2;
			}
			break;

	case GDK_Left:
			if (OglScene->__iMode__==1) //If edit mode
			{
				cam_rotx -= 0.2;
			}
			if (OglScene->__iMode__==2) //If view mode
			{
				view_rotx -= 0.2;
			}
			break;

	case GDK_Right:
			if (OglScene->__iMode__==1) //If edit mode
			{
				cam_rotx += 0.2;
			}
			if (OglScene->__iMode__==2) //If view mode
			{
				view_rotx += 0.2;
			}
			break;

	case GDK_f:
		if (OglScene->__iFullScreen__==0)
		{
			gtk_window_fullscreen (GTK_WINDOW (wOgl));
			OglScene->__iFullScreen__=1;
		}else{
			gtk_window_unfullscreen (GTK_WINDOW (wOgl));
			OglScene->__iFullScreen__=0;
		}
	case GDK_p:
		g_print ("LOG-OglPanel.c: View: x:%f y:%f z:%f Cam: x:%f y:%f z:%f \n", (float) view_rotx, (float) view_roty, (float) view_rotz, (float) cam_rotx, (float) cam_roty, (float) cam_rotz);
		break;

	/*case GDK_8:
        iLightx=iLightx+0.5;
		break;

	case GDK_2:
        iLightx=iLightx-0.5;
		break;

	case GDK_6:
        iLighty=iLighty+0.5;
		break;

	case GDK_4:
        iLighty=iLighty-0.5;
		break;*/

	default:
		return FALSE;
	}
	return TRUE;
}



void
OnStopGL                 ( GtkWidget * widget,
                           gpointer data      ){
	OglScene->__alive__ = FALSE;
	#ifdef DEBUG_OGLPAN
		g_print ("LOG-OglPanel.c: Close Event.\n");
	#endif
	gtk_widget_destroy (widget);
	if (OglScene->__only__==1)
		{
		gtk_main_quit();
		}
	return;
}

void
realize                  ( GtkWidget * widget,
                           gpointer data      ){
	GdkGLContext *glcontext = gtk_widget_get_gl_context (widget);
	GdkGLDrawable *gldrawable = gtk_widget_get_gl_drawable (widget);
	float fYmax,fXmax;



	int iFlagX, iFlagY; /* Loop variables */

	/*** OpenGL BEGIN ***/
	if (!gdk_gl_drawable_gl_begin (gldrawable, glcontext))
		return;

    if (!checkVersion())
    {
       /* OglScene->__alive__ = FALSE;*/
        #ifdef DEBUG_OGLPAN
            g_print ("LOG-OglPanel.c::realize: Close Event.\n");
        #endif
        /*gtk_widget_destroy (widget);
        exit(1);*/
    }
    
	glEnable (GL_TEXTURE_2D);
	glGenLists (10);

	#ifdef DEBUG_OGLPAN
		g_print ("LOG-OglPanel.c::realize: Loading starting texture for object %i.\n",RESERVED_OBJECT);
	#endif
	/*ptTexA = LoadJpegTextures ((char *) OglScene->sCheminA);*/
	/*OglScene->__iCurrentPlay__=RESERVED_OBJECT;*/
	#ifdef DEBUG_OGLPAN
		g_print ("LOG-OglPanel.c::realize: Loading starting texture %s.\n",OglScene->CurrentPlay);
	#endif
	strcpy(OglScene->CurrentPlay,"data/logo.png");
	
    OglObject[RESERVED_OBJECT].hTexture = Img2Tex_SDL(OglScene->CurrentPlay, GL_LINEAR_MIPMAP_LINEAR,GL_LINEAR,GL_CLAMP_TO_EDGE,GL_CLAMP_TO_EDGE);

    /*OglObject[RESERVED_OBJECT].sPath = malloc(sizeof(char)     * (999+1))          ;*/

    strncpy(OglObject[  RESERVED_OBJECT].sPath,
                            OglScene->CurrentPlay,
                            strlen(OglScene->CurrentPlay));

    OglObject[RESERVED_OBJECT].sPath[strlen(OglScene->CurrentPlay)] = '\0';
    OglObject[RESERVED_OBJECT].fAngle = 0.0;
	/*AddNewGLObject();*/
    OglScene->__iCurrentPlay__   = RESERVED_OBJECT   ;
	OglScene->iNbrEntryObj       = 0    ;
	#ifdef DEBUG_OGLPAN
		g_print ("LOG-OglPanel.c::realize: Generate button.\n");
	#endif
	nCdBox (6);
    nFxBox (7);
	LdToolbar(j3denv->__sTbCurSkin__);
    ToolbarFx  = malloc(sizeof(FxStruct)     * 10)          ;


	#ifdef DEBUG_OGLPAN
		g_print ("LOG-OglPanel.c::realize: Generate plane model.\n");
	#endif
	nPlaneTeam (3);
	#ifdef DEBUG_OGLPAN
		g_print ("LOG-OglPanel.c::realize: Generate cube model.\n");
	#endif
    nCubeTeam (1);
	#ifdef DEBUG_OGLPAN
		g_print ("LOG-OglPanel.c::realize: Generate cd box model.\n");
	#endif
	nCdBox (4);

	#ifdef DEBUG_OGLPAN
		g_print ("LOG-OglPanel.c::realize: Generate Rectangle model.\n");
	#endif
	nRectTeam (5);

	#ifdef DEBUG_OGLPAN
		g_print ("LOG-OglPanel.c::realize: Initialize sponsors sytem.\n");
	#endif
	/*font_sponsors.iFontSize=font_sponsors.iFontSize*2;*/
	if (OglScene->__ActSpons__==1)
	{
		ConfigureText(widget);
	}

	#ifdef DEBUG_OGLPAN
		g_print ("LOG-OglPanel.c::realize: Initialize flag model sytem.\n");
	#endif

    /* Loop Through The X Plane */
    for ( iFlagX = 0; iFlagX < 23; iFlagX++ )
        {
    /* Loop Through The Y Plane */
	    for ( iFlagY = 0; iFlagY < 23; iFlagY++ )
                {
		    /* Apply The Wave To Our Mesh */
		    points[iFlagX][iFlagY][0] = ( float )( iFlagX/ 5.0 ) - 4.5;
		    points[iFlagX][iFlagY][1] = ( float )( ( iFlagY / 5.0f ) - 4.5f );
		    points[iFlagX][iFlagY][2] = ( float )( sin( ( ( ( iFlagX / 5.0f ) * 40.0f ) / 360.0f ) * 3.141592654 * 2.0f ) );
                }
        }
		fYmax = (2*cam_rotz)* tan((60*DIG_2_RAD) / 2);
	 	fXmax = 8/6 * fYmax;
		OglScene->__dColumnSize__=(double)widget->allocation.width/(double)fXmax;
		#ifdef DEBUG_OGLPAN
			g_print ("LOG-OglPanel.c: Taille en pixel d'une colonne: %f.\n",OglScene->__dColumnSize__);
		#endif

	gdk_gl_drawable_gl_end (gldrawable);


	/**triggering the timer**/
	if (timerFPS == NULL)
		timerFPS = g_timer_new ();
	g_timer_start (timerFPS);

	return;
}




gboolean
configure_event          ( GtkWidget * widget,
                           GdkEventConfigure * event,
                           gpointer data      ){
	GdkGLContext *glcontext = gtk_widget_get_gl_context (widget);
	GdkGLDrawable *gldrawable = gtk_widget_get_gl_drawable (widget);

	/*** OpenGL BEGIN ***/
	if (!gdk_gl_drawable_gl_begin (gldrawable, glcontext))
		return FALSE;
  	float sine = sin (0.5 * VIEW_INIT_ANGLE * DIG_2_RAD);

	#ifdef DEBUG_OGLPAN
	/*	float VP[4];
		glGetFloatv(GL_VIEWPORT,VP);
		g_print("LOG-OglPanel.c: Float 1: %f\n",VP[0]);
		g_print("LOG-OglPanel.c: Float 2: %f\n",VP[1]);
		g_print("LOG-OglPanel.c: Float 3: %f\n",VP[2]);
		g_print("LOG-OglPanel.c: Float 4: %f\n",VP[3]);
		printf("LOG-OglPanel.c: Viewport Width: %d Height: %d\n", (int)widget->allocation.width, (int)widget->allocation.height);*/
	#endif

	view_quat[0] = VIEW_INIT_AXIS_X * sine;
	view_quat[1] = VIEW_INIT_AXIS_Y * sine;
	view_quat[2] = VIEW_INIT_AXIS_Z * sine;
	view_quat[3] = cos (0.5 * VIEW_INIT_ANGLE * DIG_2_RAD);
	view_scale = 1.0;
	SW=widget->allocation.width;
	SH=widget->allocation.height;
	glViewport (0, 0, widget->allocation.width,
		    widget->allocation.height);
	glMatrixMode (GL_PROJECTION);
	glLoadIdentity ();
	/**OpenGLWay **/
	/*glFrustum (-1.0, 1.0, -h, h, 5.0, 80.0);*/
	/**Rad GLU Way**/
	gluPerspective(45.0f, widget->allocation.width/widget->allocation.height, 0.1f, 100.0f);
	glMatrixMode (GL_MODELVIEW);
	glLoadIdentity ();
	if (OglScene->__ActSpons__==1)
	{
        ConfigureText(widget);
	}
	gdk_gl_drawable_gl_end (gldrawable);
	return TRUE;
}



void
SetEditMode (GtkWidget * widget,
             gpointer data)
{
	#ifdef DEBUG_OGLPAN
		g_print("LOG-OglPanel.c: Edit mode\n");
	#endif
	OglScene->__iMode__=0;
	return;
}
void
SetCamMode (GtkWidget * widget, gpointer data)
{
	#ifdef DEBUG_OGLPAN
		g_print("LOG-OglPanel.c: Camera mode\n");
	#endif
	OglScene->__LastClicked__=0;
	OglScene->__iMode__=1;
	return;
}
void
SetViewMode (GtkWidget * widget, gpointer data)
{
	#ifdef DEBUG_OGLPAN
		g_print("LOG-OglPanel.c: View mode\n");
	#endif
	OglScene->__LastClicked__=0;
	OglScene->__iMode__=2;
	return;
}

void
SetFullscreen (GtkWidget * widget, gpointer data)
{
		if (OglScene->__iFullScreen__==0)
		{

		    /*font_sponsors.iFontSize=font_sponsors.iFontSize*2;*/
			gtk_window_fullscreen (GTK_WINDOW (wOgl));
			OglScene->__iFullScreen__=1;

		}else{

			/*font_sponsors.iFontSize=(int)font_sponsors.iFontSize/2;*/
			gtk_window_unfullscreen (GTK_WINDOW (wOgl));
			OglScene->__iFullScreen__=0;

		}
	return;
}

void
SetToolbarMode (GtkWidget * widget, gpointer data)
{
	#ifdef DEBUG_OGLPAN
		g_print("LOG-OglPanel.c: Toolbar mode\n");
	#endif
	if (OglScene->__iHideToolbar__==0)
	{
	    OglScene->__iHideToolbar__=1;
    }else{
	    OglScene->__iHideToolbar__=0;
    }
	return;
}
void
timeout_remove (GtkWidget * widget)
{
	#ifdef DEBUG_OGLPAN
		g_print ("LOG-OglPanel.c: On remove le timeout.\n");
	#endif
	if (timeout_id != 0)
	{
		g_source_remove (timeout_id);
		timeout_id = 0;
	}
}

gboolean
timeout (GtkWidget * widget)
{
	/* Invalidate the whole window. */
	if (OglScene->__alive__)
	{
		gdk_window_invalidate_rect (widget->window,
					    &widget->allocation, FALSE);
		/* Update synchronously. */
		gdk_window_process_updates (widget->window, FALSE);
		return TRUE;
	}
	timeout_remove (widget);
	return FALSE;
}

void
timeout_add (GtkWidget * widget)
{
	if (timeout_id == 0)
	{
		timeout_id =
			g_timeout_add (TIMEOUT_INTERVAL,
				       (GSourceFunc) timeout, widget);
	}
}
gboolean
map_event (GtkWidget * widget, GdkEventAny * event, gpointer data)
{
	if ((animate) && (OglScene->__alive__))
	{
		timeout_add (widget);
	}
	return TRUE;
}

gboolean
unmap_event (GtkWidget * widget, GdkEventAny * event, gpointer data)
{
	timeout_remove (widget);

	return TRUE;
}
gboolean
visibility_notify_event (GtkWidget * widget,
			             GdkEventVisibility * event,
                         gpointer data)
{
	if (animate)
	{
		if (event->state == GDK_VISIBILITY_FULLY_OBSCURED)
			timeout_remove (widget);
		else
			timeout_add (widget);
	}

	return TRUE;
}
/***
 *** Creates the popup menu to be displayed.
 ***/
/* For popup menu. */
static gboolean
button_press_event_popup_menu (GtkWidget      *widget,
			       GdkEventButton *event,
			       gpointer        data)
{
	#ifdef DEBUG_OGLPAN
  		g_print ("LOG-OglPanel.c: %s: \"button_press_event_popup\" \n", gtk_widget_get_name (widget));
	#endif
	if (event->button == 3)
    {
		#ifdef DEBUG_OGLPAN
   			g_print ("LOG-OglPanel.c: button 3\n");
		#endif
		gtk_menu_popup (GTK_MENU (widget), NULL, NULL, NULL, NULL,
						event->button, event->time);
		return TRUE;
    }
	return FALSE;
}
static GtkWidget *
create_popup_menu (GtkWidget *drawing_area)
{
  GtkWidget *menu;
  GtkWidget *menu_item;

  menu = gtk_menu_new ();

  /* EDITION */
  /*menu_item = gtk_menu_item_new_with_label ("Mode edition");
  gtk_menu_shell_append (GTK_MENU_SHELL (menu), menu_item);
  g_signal_connect (G_OBJECT (menu_item), "activate",
		    G_CALLBACK (SetEditMode), NULL);
  gtk_widget_show (menu_item);*/

  /* Camera */
  /*menu_item2 = gtk_menu_item_new_with_label ("Mode camera");
  gtk_menu_shell_append (GTK_MENU_SHELL (menu), menu_item2);
  g_signal_connect (G_OBJECT (menu_item2), "activate",
		    G_CALLBACK (SetCamMode), NULL);
  gtk_widget_show (menu_item2);*/

  /* View */
  /*menu_item3 = gtk_menu_item_new_with_label ("Mode point de vue");
  gtk_menu_shell_append (GTK_MENU_SHELL (menu), menu_item3);
  g_signal_connect (G_OBJECT (menu_item3), "activate",
		    G_CALLBACK (SetViewMode), NULL);
  gtk_widget_show (menu_item3);*/

  /* Quit */
  menu_item = gtk_menu_item_new_with_label ("Show / Hide button");
  gtk_menu_shell_append (GTK_MENU_SHELL (menu), menu_item);
  g_signal_connect (G_OBJECT (menu_item), "activate",
		    G_CALLBACK (SetToolbarMode), NULL);
  gtk_widget_show (menu_item);

  menu_item = gtk_menu_item_new_with_label ("Full screen / Windowed");
  gtk_menu_shell_append (GTK_MENU_SHELL (menu), menu_item);
  g_signal_connect (G_OBJECT (menu_item), "activate",
		    G_CALLBACK (SetFullscreen), NULL);
  gtk_widget_show (menu_item);

  /*menu_item = gtk_menu_item_new_with_label ("Full screen");
  gtk_menu_shell_append (GTK_MENU_SHELL (menu), menu_item);
  g_signal_connect (G_OBJECT (menu_item), "activate",
		    G_CALLBACK (SetViewMode), NULL);
  gtk_widget_show (menu_item);*/
  return menu;
}

GtkWidget *
StartOGL (void)
{
    GtkWidget *menu;
	GtkWidget *drawing_area  ;
	GdkGLConfig *glconfig;
	/*gint major, minor;*/
 	GladeXML *xml;

	OglScene->__alive__ = TRUE;
  	xml = glade_xml_new (j3denv->__sGladeOgl__, NULL, NULL);
    /*
    * Connect the signals in the interface.
    */
  	glade_xml_signal_autoconnect (xml);
    /*
    * Top-level window.
    */
  	wOgl = glade_xml_get_widget (xml, "window1");
  	gtk_container_set_reallocate_redraws (GTK_CONTAINER (wOgl), TRUE);
    /*
	* Drawing area.
	*/
  	drawing_area = glade_xml_get_widget (xml, "drawingarea1");

	/** Configure OpenGL-capable visual.*/
	/* Try double-buffered visual */
	glconfig =
		gdk_gl_config_new_by_mode (GDK_GL_MODE_RGB | GDK_GL_MODE_DEPTH | GDK_GL_MODE_DOUBLE);
	if (glconfig == NULL)
	{
		g_print ("*** Cannot find the double-buffered visual.\n");
		g_print ("*** Trying single-buffered visual.\n");
		/* Try single-buffered visual */
		glconfig =
			gdk_gl_config_new_by_mode (GDK_GL_MODE_RGB |
						   GDK_GL_MODE_DEPTH);
		if (glconfig == NULL)
		{
			g_print ("*** No appropriate OpenGL-capable visual found.\n");
			exit (1);
		}
	}
	/* Set OpenGL-capability to the widget. */
	gtk_widget_set_gl_capability (drawing_area, glconfig, NULL, TRUE,
				      GDK_GL_RGBA_TYPE);

	gtk_widget_add_events (drawing_area,
			       GDK_BUTTON1_MOTION_MASK |
			       GDK_BUTTON2_MOTION_MASK | GDK_BUTTON_PRESS_MASK
			       | GDK_VISIBILITY_NOTIFY_MASK );

			       /*| GDK_POINTER_MOTION_MASK*/
    menu = create_popup_menu (drawing_area);
    g_signal_connect_swapped (G_OBJECT (drawing_area), "button_press_event",
    G_CALLBACK (button_press_event_popup_menu), menu);
	gtk_widget_show (drawing_area);

	return wOgl;

}

void nFlagTeam( int x, int y, float f_x, float f_y,float f_xb,float f_yb)
{

    /* Start Drawing Our Quads */
    glBegin( GL_QUADS );
    /* Loop Through The X Plane 0-44 (45 Points) */
    for( x = 0; x < 22; x++ )
	{
	    /* Loop Through The Y Plane 0-44 (45 Points) */
	    for( y = 0; y < 22; y++ )
		{
		    /* Create A Floating Point X Value */
		    f_x = ( float )x / 22.0f;
		    /* Create A Floating Point Y Value */
		    f_y = ( float )y / 22.0f;
		    /* Create A Floating Point Y Value+0.0227f */
		    f_xb = ( float )( x + 1 ) / 22.0f;
		    /* Create A Floating Point Y Value+0.0227f */
		    f_yb = ( float )( y + 1 ) / 22.0f;

		    /* First Texture Coordinate (Bottom Left) */
		    /*glTexCoord2f( f_x, f_yb );*/

		    glTexCoord2f( f_x, f_y );
		    glVertex3f( points[x][y][0], points[x][y][1],
				points[x][y][2] );

		    /* Second Texture Coordinate (Top Left) */
		    glTexCoord2f( f_x, f_yb );
		    glVertex3f( points[x][y + 1][0], points[x][y + 1][1],
				points[x][y + 1][2] );

		    /* Third Texture Coordinate (Top Right) */
		    glTexCoord2f( f_xb, f_yb );
		    glVertex3f( points[x + 1][y + 1][0],
				points[x + 1][y + 1][1],
				points[x + 1][y + 1][2] );

		    /* Fourth Texture Coordinate (Bottom Right) */
		    glTexCoord2f( f_xb, f_y );
		    glVertex3f( points[x + 1][y][0], points[x + 1][y][1],
				points[x + 1][y][2] );
                }
        }
    glEnd( );
return;

}

gboolean
expose_event             ( GtkWidget * widget,
                           GdkEventExpose * event,
                           gpointer data      ){
	GdkGLContext *glcontext = gtk_widget_get_gl_context (widget);
	GdkGLDrawable *gldrawable = gtk_widget_get_gl_drawable (widget);

	/*** OpenGL BEGIN ***/
	if (!gdk_gl_drawable_gl_begin (gldrawable, glcontext))
		return FALSE;
    /*float iMySpeed=(float)OglScene->__speed__/10;*/

	/**Draw eSportsPanel **/
	/*RenderToTexture(widget);*/
	draw_object(GL_RENDER,widget);

	/*RenderGlow(20,widget);*/

	if (gdk_gl_drawable_is_double_buffered (gldrawable))
	{gdk_gl_drawable_swap_buffers (gldrawable);}
	else
	{glFlush ();}
	gdk_gl_drawable_gl_end (gldrawable);

    /* Used To Slow Down The Wave (Every 2nd Frame Only) */
    #ifdef DEBUG_OGLPAN_OGLPAN
               /* g_print ("Log: OglPanel.c::expose_event: OglScene->__speed__ -> %f \n",  OglScene->__speed__);
        		g_print ("Log: OglPanel.c::expose_event: (int)iMySpeed -> %i \n",  (int)iMySpeed);
        		g_print ("Log: OglPanel.c::expose_event: iTest -> %i \n",  iTest);*/
    #endif

    if( wiggle_count == 6 )
        {
	    /* Loop Through The Y Plane */
	    for( fy = 0; fy < 23; fy++ )
		{
		    /* Store Current Value One Left Side Of Wave */
		    hold = points[0][fy][2];
		    /* Loop Through The X Plane */
		    for( fx = 0; fx < 22; fx++)
			{
                            /* Current Wave Value Equals Value To The Right */
			    points[fx][fy][2] = points[fx + 1][fy][2];
                        }
		    /* Last Value Becomes The Far Left Stored Value */
		    points[22][fy][2] = hold;
                }
	    wiggle_count = 0; /* Set Counter Back To Zero */
        }
    wiggle_count++; /* Increase The Counter */

	frames++;
	seconds = g_timer_elapsed (timerFPS, NULL);
	if (seconds >= 5.0)
	{
		fps = frames / seconds;
		g_print ("%d frames in %6.3f seconds = %6.3f FPS\n", frames,
			 seconds, fps);
		g_timer_reset (timerFPS);
		frames = 0;
	}
	#ifdef DEBUG_OGLPAN
			/*printf("Seconds:  %f\n", (float)secondsFPS);*/
	#endif
	return TRUE;
}



gboolean
button_press_event       ( GtkWidget      *widget,
                           GdkEventButton *event,
                           GtkWidget      *menu){
	float    begin_x      = 0.0   ;
	float    begin_y      = 0.0   ;
	begin_x = event->x;
	begin_y = event->y;
	#ifdef DEBUG_OGLPAN
		g_print("LOG-OglPanel.c::button_press_event: iMode -> %i\n",OglScene->__iMode__);
	#endif
	if (event->button == 1)
	{
		if (OglScene->__iMode__==0) //If edit mode
		{
			gl_select(event->x, SH-(event->y),widget);
			#ifdef DEBUG_OGLPAN
				printf("LOG-OglPanel.c::button_press_event: Mouse button pressed x -> %d y -> %d\n", (int)event->x, (int)event->y);
			#endif
		}
	}
	return TRUE;
 }

gboolean
motion_notify_event (GtkWidget * widget,
		     GdkEventMotion * event, gpointer data)
{
/*	GLint viewport[4];
	GLdouble mvmatrix[16], projmatrix[16];
	GLdouble wx, wy, wz; // coordonnÃÂ©es x, y, z dans le repert ogl
	glGetIntegerv(GL_VIEWPORT, viewport);
	glGetDoublev(GL_MODELVIEW_MATRIX, mvmatrix);
	glGetDoublev(GL_PROJECTION_MATRIX, projmatrix);*/
	float fYmax,fXmax;
	gboolean redraw = FALSE;
	if (event->state & GDK_BUTTON1_MASK)
	{

		/*quelque soit la position en Z de la cam, tan( fovy / 2) = Ymax / 2Z
		donc ton Ymax = 2Z* tan( fovy / 2) avec le fovy en radian.
		apres, tu sais que Ymax / Xmax = 600 / 800, donc tu peux en deduire Xmax = 8/6 * Ymax
		et la conversion depuis pixels est la suivante
		sur X : X_ogl = X_pix * Xmax/800
		sur Y : Y_ogl = Y_pix * Ymax/600*/

		if (OglScene->__iMode__==0) //If edit mode
		{
		fYmax = (2*cam_rotz)* tan((60*DIG_2_RAD) / 2);
	 	fXmax = 8/6 * fYmax;
	 	fXView = event->x * (double)fXmax/(double)widget->allocation.width;
		fYView = event->y * -(double)fYmax/(double)widget->allocation.height;
		/*gluUnProject(event->x, event->y, 0.0, mvmatrix, projmatrix, viewport, &wx, &wy, &wz);*/
		#ifdef DEBUG_OGLPAN
	 		/*g_print("LOG-OglPanel.c: MAXXogl: %f MAXYogl: %f\n",fXmax,fYmax);
	 		g_print("LOG-OglPanel.c: Xogl: %f Yogl: %f\n",fXView,fYView);*/
			/* g_print("motion_notify_event\n");*/
			/*printf("les coord du curseurs sont %f - %f\n", event->x, event->y);
			printf("les coord en z = 0.0 sont wx %f; wy %f; wz %f\n", wx, wy, wz); */
		#endif
		OglScene->__dColumnSize__=(double)widget->allocation.width/(double)fXmax;

		/**each model have different size, static compensation is a lazy solution**/
        if (OglScene->__LastClicked__>=RESERVED_OBJECT)
        {
            switch(OglScene->__model__)
            {
                case 0:
                    OglObject[OglScene->__iCurrentPlay__].x=fXView-3;
                    OglObject[OglScene->__iCurrentPlay__].y=fYView+4;
                    break;
                case 1:
                    OglObject[OglScene->__iCurrentPlay__].x=fXView-4;
                    OglObject[OglScene->__iCurrentPlay__].y=fYView+4;
                    break;
                case 2:
                    OglObject[OglScene->__iCurrentPlay__].x=fXView-3;
                    OglObject[OglScene->__iCurrentPlay__].y=fYView+4;
                    break;
                case 3:
                    OglObject[OglScene->__iCurrentPlay__].x=fXView-1.5;
                    OglObject[OglScene->__iCurrentPlay__].y=fYView+2.3;
                    break;
                case 5:
                    OglObject[OglScene->__iCurrentPlay__].x=fXView-1.5;
                    OglObject[OglScene->__iCurrentPlay__].y=fYView+2;
                default:
                    return TRUE;
            }
        }else{
                switch(OglScene->__LastClicked__)
                {
                    case 2:
                        OglScene->__aArchivePos__[0]=fXView+4;
                        OglScene->__aArchivePos__[1]=-fYView-4;
						/*g_print("LOG-OglPanel.c: fXView: %f fYView: %f\n",fXView,fYView);
						g_print("LOG-OglPanel.c: OglScene->__aArchivePos__[0]: %f OglScene->__aArchivePos__[1]: %f\n",OglScene->__aArchivePos__[0],OglScene->__aArchivePos__[1]);*/
                        break;
                    case 3:
                        OglScene->__fGroundPos__     =-fYView-2;
                        break;
                    case 5:
                        OglObject[1].x=fXView-3;
                        OglObject[1].y=fYView+5;
                        break;
                    default:
                        break;
                }
            }
		}
		redraw = TRUE;
	}
	if (redraw && !animate)
		gdk_window_invalidate_rect (widget->window,
					    &widget->allocation, FALSE);

	return TRUE;
}


void
draw_object              ( GLenum mode,
                           GtkWidget *widget ){

    /*	RenderToMotionBlurTexture(FALSE ,0);
        ShowMotionBlurTexture();*/
	/*g_print ("LOG-OglPanel.c::draw_object: OglScene.__LiveTexRefresh__ %i\n",OglScene.__LiveTexRefresh__);*/
    int iTemp=0;
	/*if (OglScene.__LiveTexRefresh__==1)
	{
        glEnable (GL_TEXTURE_2D);
        glDeleteTextures(1,&OglObject[0].hTexture);
        OglObject[0].hTexture = Img2Tex_SDL((char *) OglScene.sCheminA, GL_LINEAR_MIPMAP_LINEAR,GL_LINEAR,GL_CLAMP_TO_EDGE,GL_CLAMP_TO_EDGE);
        OglScene.__LiveTexRefresh__=0;
        #ifdef DEBUG_OGLPAN
            g_print ("LOG-OglPanel.c::draw_object: Reloading texture\n");
        #endif
	}*/

 	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
 	glMatrixMode(GL_MODELVIEW);

    glClearColor(OglScene->__aWorldColor__[0], \
                 OglScene->__aWorldColor__[1], \
                 OglScene->__aWorldColor__[2],1.0f);
	glLoadIdentity();

    /**Camera setup**/
	glPushAttrib(GL_ALL_ATTRIB_BITS);
	gluLookAt (	cam_rotx, cam_roty, cam_rotz,
				view_rotx, view_roty,view_rotz,
				0, 1, 0);
    /***************/



/**Background Object************/
if (OglObject[1].hidden==0)
{
    glPushMatrix ();
        glLoadName(5);
        /*glEnable (GL_DEPTH_TEST);*/
        glDisable (GL_TEXTURE_2D);

        glTranslatef (  OglObject[1].x,
                        OglObject[1].y,
                        OglObject[1].z);

        glColor3f    (  OglObject[1].red,
                        OglObject[1].green,
                        OglObject[1].blue);

        glBegin(GL_QUADS);
            /** lower right corner of image **/
            glVertex2f(-OglObject[1].fWidth, -OglObject[1].fHeight);
            /** lower left corner of image */
            glVertex2f( OglObject[1].fWidth, -OglObject[1].fHeight);
            /** upper left corner of image */
            glVertex2f(OglObject[1].fWidth,  OglObject[1].fHeight);
            /** upper right corner of image */
            glVertex2f(-OglObject[1].fWidth,  OglObject[1].fHeight);
        glEnd();

        glEnable (GL_TEXTURE_2D);
    glPopMatrix ();
}

/**Dynamic Object*******************/
glColor3f (255.0, 255.0, 255.0);
    switch(OglScene->__model__)
    {
        /**Cube**/
            case 0:
                glPushMatrix ();
                glEnable (GL_DEPTH_TEST);
                        /*g_print("Painting object Y %i\n",2*(RESERVED_OBJECT-iTemp));*/

                /*glTranslatef (7.5,-5,0);*/
                glTranslatef (  OglObject[OglScene->__iCurrentPlay__].x,
                                OglObject[OglScene->__iCurrentPlay__].y,
                                OglObject[OglScene->__iCurrentPlay__].z-5);
                glBindTexture (GL_TEXTURE_2D, OglObject[OglScene->__iCurrentPlay__].hTexture);
                SceneStyleAnim(&OglObject[OglScene->__iCurrentPlay__].fAngle,OglScene->__style_anim__);

                if (mode == GL_SELECT)
                {glLoadName(RESERVED_OBJECT);}
                /*if (OglScene->__LastClicked__==RESERVED_OBJECT)
                {	glDisable (GL_TEXTURE_2D);
                    glColor3f(fMyColor,0.5,1.0);
                    glPolygonMode( GL_FRONT_AND_BACK, GL_LINE );
                    glLineWidth( 3.0f );
                }*/
                /*glScalef(1.5F,1.5F,1.5F) ;*/
                glScalef(   OglScene->__fCurPlaySize__,
                            OglScene->__fCurPlaySize__,
                            OglScene->__fCurPlaySize__) ;
                glCallList (1);
                /*if (OglScene->__LastClicked__==RESERVED_OBJECT)
                {			glEnable (GL_TEXTURE_2D);
                    glColor3f(1.0,1.0,1.0);
                    glPolygonMode( GL_FRONT_AND_BACK, GL_FILL);}*/

                glPopMatrix ();




                    if (OglScene->__iActCurPlayRef__==1)
                    {
                        glPushMatrix ();
                        glEnable(GL_BLEND);
                            glLoadName(3);
                            glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
                            glColor4f(1, 1, 1, reflexpwr);
                            /*glTranslatef (  OglObject[OglScene->__iCurrentPlay__].x,
                                OglObject[OglScene->__iCurrentPlay__].y-2*OglScene->__fCurPlaySize__,
                                OglObject[OglScene->__iCurrentPlay__].z-5);*/
                            glTranslatef (  OglObject[OglScene->__iCurrentPlay__].x,
                                -OglScene->__fGroundPos__,
                                OglObject[OglScene->__iCurrentPlay__].z-5);


                            //SceneStyleAnim(iMySpeed,OglScene->__style_animA__,0);
                            glRotated(180,0,0,0);
                            SceneStyleAnim(&OglObject[OglScene->__iCurrentPlay__].fAngle,OglScene->__style_anim__);
                            glDepthMask(GL_FALSE);
                            glEnable(GL_CULL_FACE);
                            glCullFace(GL_FRONT);
                            glScalef(   OglScene->__fCurPlaySize__,
                                        OglScene->__fCurPlaySize__,
                                        OglScene->__fCurPlaySize__) ;
                            glCallList (1);
                            glDepthMask(GL_TRUE);
                            glDisable(GL_CULL_FACE);
                        glDisable(GL_BLEND);

                        glPopMatrix ();

                    }




                break;
            /**Rectangle**/
            case 1:
                glPushMatrix ();
                glEnable (GL_DEPTH_TEST);
                glTranslatef (  OglObject[OglScene->__iCurrentPlay__].x,
                                OglObject[OglScene->__iCurrentPlay__].y,
                                OglObject[OglScene->__iCurrentPlay__].z-5);
                glBindTexture (GL_TEXTURE_2D, OglObject[OglScene->__iCurrentPlay__].hTexture);
                SceneStyleAnim(&OglObject[OglScene->__iCurrentPlay__].fAngle,OglScene->__style_anim__);
                if (mode == GL_SELECT)
                {glLoadName(RESERVED_OBJECT);}
                /*if (OglScene->__LastClicked__==1)
                {	glDisable (GL_TEXTURE_2D);
                    glColor3f(fMyColor,0.5,1.0);
                    glPolygonMode( GL_FRONT_AND_BACK, GL_LINE );
                    glLineWidth( 3.0f );
                }*/

                glScalef(   OglScene->__fCurPlaySize__,
                            OglScene->__fCurPlaySize__,
                            OglScene->__fCurPlaySize__) ;
                glCallList (5);
               /*if (OglScene->__LastClicked__==1)
                {			glEnable (GL_TEXTURE_2D);
                    glColor3f(1.0,1.0,1.0);
                    glPolygonMode( GL_FRONT_AND_BACK, GL_FILL);}*/
                glPopMatrix ();

                    if (OglScene->__iActCurPlayRef__==1)
                    {
                        glPushMatrix ();
                        glEnable(GL_BLEND);
                            glLoadName(3);
                            glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
                            glColor4f(1, 1, 1, reflexpwr);
                            glTranslatef (  OglObject[OglScene->__iCurrentPlay__].x,
                                -OglScene->__fGroundPos__,
                                OglObject[OglScene->__iCurrentPlay__].z-5);
                            glRotated(180,0,0,0);
                            SceneStyleAnim(&OglObject[OglScene->__iCurrentPlay__].fAngle,OglScene->__style_anim__);
                            glDepthMask(GL_FALSE);
                            glEnable(GL_CULL_FACE);
                            glCullFace(GL_FRONT);
                            glScalef(   OglScene->__fCurPlaySize__,
                                        OglScene->__fCurPlaySize__,
                                        OglScene->__fCurPlaySize__) ;
                            glCallList (5);
                            glDepthMask(GL_TRUE);
                            glDisable(GL_CULL_FACE);
                        glDisable(GL_BLEND);

                        glPopMatrix ();

                    }



                break;

            /**FLAT**/
             case 2:
                glPushMatrix ();
                /*glTranslatef (OglObject[0].x, OglObject[0].y,OglObject[0].z);
                SceneStyleAnim(iMySpeed,OglScene->__style_animA__,0);*/
                glTranslatef (  OglObject[OglScene->__iCurrentPlay__].x,
                                OglObject[OglScene->__iCurrentPlay__].y,
                                OglObject[OglScene->__iCurrentPlay__].z-5);
                glBindTexture (GL_TEXTURE_2D, OglObject[OglScene->__iCurrentPlay__].hTexture);
                SceneStyleAnim(&OglObject[OglScene->__iCurrentPlay__].fAngle,OglScene->__style_anim__);
                if (mode == GL_SELECT)
                {glLoadName(RESERVED_OBJECT);}
                /*if (OglScene->__LastClicked__==1)
                {	glDisable (GL_TEXTURE_2D);
                    glColor3f(fMyColor,0.5,1.0);
                    glPolygonMode( GL_FRONT_AND_BACK, GL_LINE );
                    glLineWidth( 3.0f );
                }*/
                glScalef(   OglScene->__fCurPlaySize__,
                            OglScene->__fCurPlaySize__,
                            OglScene->__fCurPlaySize__) ;
                glCallList (3);
                /*if (OglScene->__LastClicked__==1)
                {			glEnable (GL_TEXTURE_2D);
                    glColor3f(1.0,1.0,1.0);
                    glPolygonMode( GL_FRONT_AND_BACK, GL_FILL);}*/
                glPopMatrix ();
                    if (OglScene->__iActCurPlayRef__==1)
                    {
                        glPushMatrix ();
                        glEnable(GL_BLEND);
                            glLoadName(3);
                            glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
                            glColor4f(1, 1, 1, reflexpwr);
                            glTranslatef (  OglObject[OglScene->__iCurrentPlay__].x,
                                -OglScene->__fGroundPos__,
                                OglObject[OglScene->__iCurrentPlay__].z-5);

                            glRotated(180,0,0,0);
                            SceneStyleAnim(&OglObject[OglScene->__iCurrentPlay__].fAngle,OglScene->__style_anim__);
                            glDepthMask(GL_FALSE);
                            glEnable(GL_CULL_FACE);
                            glCullFace(GL_FRONT);
                            glScalef(   OglScene->__fCurPlaySize__,
                                        OglScene->__fCurPlaySize__,
                                        OglScene->__fCurPlaySize__) ;
                            glCallList (3);
                            glDepthMask(GL_TRUE);
                            glDisable(GL_CULL_FACE);
                        glDisable(GL_BLEND);

                        glPopMatrix ();

                    }
                break;
            /**Flag**/
            case 3:
                glPushMatrix ();

                /*glTranslatef (OglObject[0].x, OglObject[0].y,(OglObject[0].z)-5.0);*/
                /*glTranslatef (9.5,-8.5,0);*/
                                glTranslatef (  OglObject[OglScene->__iCurrentPlay__].x,
                                OglObject[OglScene->__iCurrentPlay__].y,
                                OglObject[OglScene->__iCurrentPlay__].z);
                glRotated(180,3,0,0);
                glBindTexture (GL_TEXTURE_2D, OglObject[OglScene->__iCurrentPlay__].hTexture);
                glScalef(   OglScene->__fCurPlaySize__,
                            OglScene->__fCurPlaySize__,
                            OglScene->__fCurPlaySize__) ;
                if (mode == GL_SELECT)
                {glLoadName(RESERVED_OBJECT);}
                /*if (OglScene->__LastClicked__==RESERVED_OBJECT)
                {	glDisable (GL_TEXTURE_2D);
                    glColor3f(fMyColor,0.5,1.0);
                    glPolygonMode( GL_FRONT_AND_BACK, GL_LINE );
                    glLineWidth( 3.0f );
                }*/
                nFlagTeam(fx,fy,f_x,f_y,f_xb,f_yb);
                /*if (OglScene->__LastClicked__==RESERVED_OBJECT)
                {			glEnable (GL_TEXTURE_2D);
                    glColor3f(1.0,1.0,1.0);
                    glPolygonMode( GL_FRONT_AND_BACK, GL_FILL);}*/
                glPopMatrix ();


                if (OglScene->__iActCurPlayRef__==1)
                {
                    glPushMatrix ();
                    glLoadName(3);
                    glEnable(GL_BLEND);
                        glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
                        glColor4f(1, 1, 1, reflexpwr);

                        glTranslatef (OglObject[OglScene->__iCurrentPlay__].x,
                                      -OglScene->__fGroundPos__,
                                      OglObject[OglScene->__iCurrentPlay__].z);
                        glDepthMask(GL_FALSE);
                        glEnable(GL_CULL_FACE);
                        glCullFace(GL_FRONT);
                        glScalef(   OglScene->__fCurPlaySize__,
                                    OglScene->__fCurPlaySize__,
                                    OglScene->__fCurPlaySize__) ;
                        nFlagTeam(fx,fy,f_x,f_y,f_xb,f_yb);
                        glDepthMask(GL_TRUE);
                        glDisable(GL_CULL_FACE);
                    glDisable(GL_BLEND);
                    glPopMatrix ();
                }

                break;
            /**Nothing**/
            case 4:
                break;
            /**CDBOX**/
            case 5:
                glPushMatrix ();
                glEnable (GL_DEPTH_TEST);
                        /*g_print("Painting object Y %i\n",2*(RESERVED_OBJECT-iTemp));*/

                /*glTranslatef (7.5,-5,0);*/
                glTranslatef (  OglObject[OglScene->__iCurrentPlay__].x,
                                OglObject[OglScene->__iCurrentPlay__].y,
                                OglObject[OglScene->__iCurrentPlay__].z-5);

                SceneStyleAnim(&OglObject[OglScene->__iCurrentPlay__].fAngle,OglScene->__style_anim__);
                /*g_print("Painting object %i OglScene->iNbrEntryObj+RESERVED_OBJECT %i OglScene->__iCurrentPlay__ %i\n",iTemp,OglScene->iNbrEntryObj+RESERVED_OBJECT,OglScene->__iCurrentPlay__);*/
                glBindTexture (GL_TEXTURE_2D, OglObject[OglScene->__iCurrentPlay__].hTexture);
                /* g_print("Current play: %i\n",OglScene->__iCurrentPlay__);*/

                if (mode == GL_SELECT)
                {glLoadName(RESERVED_OBJECT);}
                /*if (OglScene->__LastClicked__==RESERVED_OBJECT)
                {	glDisable (GL_TEXTURE_2D);
                    glColor3f(fMyColor,0.5,1.0);
                    glPolygonMode( GL_FRONT_AND_BACK, GL_LINE );
                    glLineWidth( 3.0f );
                }*/
                glScalef(   OglScene->__fCurPlaySize__,
                            OglScene->__fCurPlaySize__,
                            OglScene->__fCurPlaySize__) ;
                glCallList (4);
                /*if (OglScene->__LastClicked__==RESERVED_OBJECT)
                {			glEnable (GL_TEXTURE_2D);
                    glColor3f(1.0,1.0,1.0);
                    glPolygonMode( GL_FRONT_AND_BACK, GL_FILL);}*/

                glPopMatrix ();

                    if (OglScene->__iActCurPlayRef__==1)
                    {
                        glPushMatrix ();
                        glEnable(GL_BLEND);
                            glLoadName(3);
                            glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
                            glColor4f(1, 1, 1, reflexpwr);
                            glTranslatef (  OglObject[OglScene->__iCurrentPlay__].x,
                                -OglScene->__fGroundPos__,
                                OglObject[OglScene->__iCurrentPlay__].z-5);

                            glRotated(180,0,0,0);
                            SceneStyleAnim(&OglObject[OglScene->__iCurrentPlay__].fAngle,OglScene->__style_anim__);
                            glDepthMask(GL_FALSE);
                            glEnable(GL_CULL_FACE);
                            glCullFace(GL_FRONT);
                            glScalef(   OglScene->__fCurPlaySize__,
                                        OglScene->__fCurPlaySize__,
                                        OglScene->__fCurPlaySize__) ;
                            glCallList (4);
                            glDepthMask(GL_TRUE);
                            glDisable(GL_CULL_FACE);
                        glDisable(GL_BLEND);

                        glPopMatrix ();

                    }
                      /*  if (mode != GL_SELECT)
                        {
                            glPushMatrix ();
                            glEnable(GL_BLEND);
                            glEnable (GL_DEPTH_TEST);
                                glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
                                glTranslatef (  OglObject[OglScene->__iCurrentPlay__].x,
                                                OglObject[OglScene->__iCurrentPlay__].y-(1.8*OglScene->__fCurPlaySize__),
                                                OglObject[OglScene->__iCurrentPlay__].z-5);

                                SceneStyleAnim(&OglObject[OglScene->__iCurrentPlay__].fAngle,OglScene->__style_animA__);
                                glRotated(180,0,0,0);
                                glScalef(   OglScene->__fCurPlaySize__,
                                            OglScene->__fCurPlaySize__,
                                            OglScene->__fCurPlaySize__) ;
                                glDepthMask(GL_TRUE);
                                glEnable(GL_CULL_FACE);
                                glCullFace(GL_FRONT);
                                glBegin(GL_QUADS);
                                    glColor4f(  1,
                                                1,
                                                1,reflexpwr);

                                    glTexCoord2f (0.0f, 1.0f);
                                    glVertex3f(-0.9, -0.9,  0.03);
                                    glTexCoord2f (1.0f, 1.0f);
                                    glVertex3f ( 0.9, -0.9,  0.03);
                                    glColor4f(1, 1, 1, 0);
                                    glTexCoord2f (1.0f, 0.0f);
                                    glVertex3f( 0.9,  0.9,  0.03);
                                    glTexCoord2f (0.0f,0.0f);
                                    glVertex3f(-0.9,  0.9,  0.03);

                                    glColor4f(1.0,1.0,1.0,0.8);
                                    glTexCoord2f (1.0f, 1.0f);
                                    glVertex3f(-0.9, -0.9, -0.03);

                                    glColor4f(0.2,0.2,0.2,0.0);
                                    glTexCoord2f (1.0f, 0.0f);
                                    glVertex3f(-0.9,  0.9, -0.03);

                                    glColor4f(0.2,0.2,0.2,0.0);
                                    glTexCoord2f (0.0f,0.0f);
                                    glVertex3f( 0.9,  0.9, -0.03);

                                    glColor4f(1.0,1.0,1.0,0.8);
                                    glTexCoord2f (0.0f,1.0f);
                                    glVertex3f( 0.9, -0.9, -0.03);

                                    glEnd();
                                    glDisable (GL_TEXTURE_2D);

                                    glColor4f(0.2,0.2,0.2,0.8);

                                    glBegin(GL_QUADS);

                                        glVertex3f(-0.9,  0.9, -0.03);
                                        glVertex3f(-0.9,  0.9,  0.03);
                                        glVertex3f( 0.9,  0.9,  0.03);
                                        glVertex3f( 0.9,  0.9, -0.03);

                                        glVertex3f(-0.9, -0.9, -0.03);
                                        glVertex3f( 0.9, -0.9, -0.03);
                                        glVertex3f( 0.9, -0.9,  0.03);
                                        glVertex3f(-0.9, -0.9,  0.03);

                                        glColor4f(0.2,0.2,0.2,0.8);
                                        glVertex3f( 0.9, -0.9, -0.03);
                                        glColor4f(0.2,0.2,0.2,0.0);
                                        glVertex3f( 0.9,  0.9, -0.03);
                                        glColor4f(0.2,0.2,0.2,0.0);
                                        glVertex3f( 0.9,  0.9,  0.03);
                                        glColor4f(0.2,0.2,0.2,0.8);
                                        glVertex3f( 0.9, -0.9,  0.03);

                                        glColor4f(0.2,0.2,0.2,0.8);
                                        glVertex3f(-0.9, -0.9, -0.03);
                                        glColor4f(0.2,0.2,0.2,0.8);
                                        glVertex3f(-0.9, -0.9,  0.03);
                                        glColor4f(0.2,0.2,0.2,0.0);
                                        glVertex3f(-0.9,  0.9,  0.03);
                                        glColor4f(0.2,0.2,0.2,0.0);
                                        glVertex3f(-0.9,  0.9, -0.03);
                                    glEnd();
                                    glEnable (GL_TEXTURE_2D);
                                glDepthMask(GL_TRUE);
                                glDisable(GL_CULL_FACE);
                            glDisable(GL_BLEND);

                            glPopMatrix ();

                        }*/
                /*if (mode != GL_SELECT)
                    {
                        glPushMatrix ();
                        glEnable(GL_BLEND);
                        glEnable (GL_DEPTH_TEST);
                            glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
                            glColor4f(1, 1, 1, reflexpwr);

                            glTranslatef (  OglObject[OglScene->__iCurrentPlay__].x,
                                            OglObject[OglScene->__iCurrentPlay__].y-2.7,
                                            OglObject[OglScene->__iCurrentPlay__].z);
                            glRotated(180,0,0,0);
                            glDepthMask(GL_FALSE);
                            glEnable(GL_CULL_FACE);
                            glCullFace(GL_FRONT);
                            glScalef(1.5F,1.5F,1.5F) ;
                            SceneStyleAnim(&OglObject[OglScene->__iCurrentPlay__].fAngle,OglScene->__style_animA__);
                               glBegin(GL_QUADS);
                                    glColor4f(  1,
                                                1,
                                                1,reflexpwr);

                                    glTexCoord2f (0.0f, 1.0f);
                                    glVertex3f(-0.9, -0.9,  0.03);
                                    glTexCoord2f (1.0f, 1.0f);
                                    glVertex3f ( 0.9, -0.9,  0.03);
                                    glColor4f(  OglScene->__aWorldColor__[0],
                                                OglScene->__aWorldColor__[1],
                                                OglScene->__aWorldColor__[2], 0);

                                    glTexCoord2f (1.0f, 0.0f);
                                    glVertex3f( 0.9,  0.9,  0.03);
                                    glTexCoord2f (0.0f,0.0f);
                                    glVertex3f(-0.9,  0.9,  0.03);

                                    glColor4f(1, 1, 1, reflexpwr);
                                    glTexCoord2f (1.0f, 1.0f);
                                    glVertex3f(-0.9, -0.9, -0.03);

                                    glColor4f(  OglScene->__aWorldColor__[0],
                                                OglScene->__aWorldColor__[1],
                                                OglScene->__aWorldColor__[2], 0);
                                    glTexCoord2f (1.0f, 0.0f);
                                    glVertex3f(-0.9,  0.9, -0.03);

                                    glColor4f(  OglScene->__aWorldColor__[0],
                                                OglScene->__aWorldColor__[1],
                                                OglScene->__aWorldColor__[2], 0);
                                    glTexCoord2f (0.0f,0.0f);
                                    glVertex3f( 0.9,  0.9, -0.03);

                                    glColor4f(1, 1, 1, reflexpwr);
                                    glTexCoord2f (0.0f,1.0f);
                                    glVertex3f( 0.9, -0.9, -0.03);

                                glEnd();

                                glDisable (GL_TEXTURE_2D);
                                glColor4f(0.2,0.2,0.2,0.8);
                                    glBegin(GL_QUADS);


                                        glVertex3f(-0.9,  0.9, -0.03);
                                        glVertex3f(-0.9,  0.9,  0.03);
                                        glVertex3f( 0.9,  0.9,  0.03);
                                        glVertex3f( 0.9,  0.9, -0.03);
                                        glVertex3f(-0.9, -0.9, -0.03);
                                        glVertex3f( 0.9, -0.9, -0.03);
                                        glVertex3f( 0.9, -0.9,  0.03);
                                        glVertex3f(-0.9, -0.9,  0.03);



                                        glColor4f(0.2,0.2,0.2,0.8);
                                        glVertex3f( 0.9, -0.9, -0.03);
                                        glColor4f(0.2,0.2,0.2,0.0);
                                        glVertex3f( 0.9,  0.9, -0.03);
                                        glColor4f(0.2,0.2,0.2,0.0);
                                        glVertex3f( 0.9,  0.9,  0.03);
                                        glColor4f(0.2,0.2,0.2,0.8);
                                        glVertex3f( 0.9, -0.9,  0.03);

                                        glColor4f(0.2,0.2,0.2,0.8);
                                        glVertex3f(-0.9, -0.9, -0.03);
                                        glColor4f(0.2,0.2,0.2,0.8);
                                        glVertex3f(-0.9, -0.9,  0.03);
                                        glColor4f(0.2,0.2,0.2,0.0);
                                        glVertex3f(-0.9,  0.9,  0.03);
                                        glColor4f(0.2,0.2,0.2,0.0);
                                        glVertex3f(-0.9,  0.9, -0.03);
                                    glEnd();
                                    glEnable (GL_TEXTURE_2D);
                            glDepthMask(GL_TRUE);
                            glDisable(GL_CULL_FACE);
                        glDisable(GL_BLEND);

                        glPopMatrix ();

                    }*/
                break;
        }
/**Archive Mode **/
if (OglScene->__iArchiveMode__==1)
{
				iArchInd=0;
                for(iTemp=RESERVED_OBJECT+OglScene->iNbrEntryObj-1;iTemp>=RESERVED_OBJECT;iTemp--)
                {

                    if (iTemp!=OglScene->__iCurrentPlay__)
                    {
						if (OglObject[iTemp].hidden!=1)
						{
							glPushMatrix ();
							glLoadName(2);
							glEnable (GL_DEPTH_TEST);
							glDisable(GL_BLEND);
								glColor3f (255.0, 255.0, 255.0);
								/*g_print("Archive painting:  %i\n",iTemp);*/

								/**iTemp mean Start of drawing - we draw from right to left**/
								/*glTranslatef ((iTemp-RESERVED_OBJECT)*(0.3*OglScene->__fArchiveSize__),
											  -5.5,
											  0.0);*/
								glTranslatef ((OglScene->__aArchivePos__[0]-iArchInd)*(0.3*OglScene->__fArchiveSize__),
											 -OglScene->__aArchivePos__[1],
											  OglScene->__aArchivePos__[2]);
								glRotated(230,0,10,0);
								glBindTexture (GL_TEXTURE_2D, OglObject[iTemp].hTexture);
								glScalef(   OglScene->__fArchiveSize__,
											OglScene->__fArchiveSize__,
											OglScene->__fArchiveSize__) ;
								glCallList (4);
								/*g_print(": %s\n",OglObject[iTemp].sPath);*/
							glPopMatrix ();
						
                        if (OglScene->__iActArchRef__==1)
                        {
							
                            glPushMatrix ();
                            glEnable(GL_BLEND);
                            glEnable (GL_DEPTH_TEST);

                                glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);


                            glTranslatef ((OglScene->__aArchivePos__[0]-iArchInd)*(0.3*OglScene->__fArchiveSize__),
                                          -OglScene->__aArchivePos__[1]-(1.8*OglScene->__fArchiveSize__) ,
                                          0.0);
                                //-(1.8*OglScene->__fArchiveSize__)
                                glRotated(230,0,10,0);
                                glRotated(180,0,0,0);
                                glScalef(   OglScene->__fArchiveSize__,
                                            OglScene->__fArchiveSize__,
                                            OglScene->__fArchiveSize__) ;
                                glDepthMask(GL_TRUE);
                                glEnable(GL_CULL_FACE);
                                glCullFace(GL_FRONT);
                                glBegin(GL_QUADS);
                                    glColor4f(  1,
                                                1,
                                                1,reflexpwr);

                                    glTexCoord2f (0.0f, 1.0f);
                                    glVertex3f(-0.9, -0.9,  0.03);
                                    glTexCoord2f (1.0f, 1.0f);
                                    glVertex3f ( 0.9, -0.9,  0.03);
                                    glColor4f(1, 1, 1, 0);
                                    glTexCoord2f (1.0f, 0.0f);
                                    glVertex3f( 0.9,  0.9,  0.03);
                                    glTexCoord2f (0.0f,0.0f);
                                    glVertex3f(-0.9,  0.9,  0.03);

                                    glColor4f(1, 1, 1, 1);
                                    /**# Back Face**/
                                    glTexCoord2f (1.0f, 1.0f);
                                    glVertex3f(-0.9, -0.9, -0.03);
                                    glTexCoord2f (1.0f, 0.0f);
                                    glVertex3f(-0.9,  0.9, -0.03);
                                    glTexCoord2f (0.0f,0.0f);
                                    glVertex3f( 0.9,  0.9, -0.03);
                                    glTexCoord2f (0.0f,1.0f);
                                    glVertex3f( 0.9, -0.9, -0.03);

                                    glEnd();
                                    glDisable (GL_TEXTURE_2D);
                                    glColor4f(0.2,0.2,0.2,0.8);

                                    glBegin(GL_QUADS);
                                        /**# Top Face**/
                                       /* glVertex3f(-0.9,  0.9, -0.03);
                                        glVertex3f(-0.9,  0.9,  0.03);
                                        glVertex3f( 0.9,  0.9,  0.03);
                                        glVertex3f( 0.9,  0.9, -0.03);*/
                                            /** # Bottom Face**/
                                        /*glVertex3f(-0.9, -0.9, -0.03);
                                        glVertex3f( 0.9, -0.9, -0.03);
                                        glVertex3f( 0.9, -0.9,  0.03);
                                        glVertex3f(-0.9, -0.9,  0.03);*/



                                        /**# Right face**/
                                        glColor4f(0.2,0.2,0.2,0.8);
                                        glVertex3f( 0.9, -0.9, -0.03);
                                        glColor4f(0.2,0.2,0.2,0.0);
                                        glVertex3f( 0.9,  0.9, -0.03);
                                        glColor4f(0.2,0.2,0.2,0.0);
                                        glVertex3f( 0.9,  0.9,  0.03);
                                        glColor4f(0.2,0.2,0.2,0.8);
                                        glVertex3f( 0.9, -0.9,  0.03);

                                        /**# Left Face**/
                                        glColor4f(0.2,0.2,0.2,0.8);
                                        glVertex3f(-0.9, -0.9, -0.03);
                                        glColor4f(0.2,0.2,0.2,0.8);
                                        glVertex3f(-0.9, -0.9,  0.03);
                                        glColor4f(0.2,0.2,0.2,0.0);
                                        glVertex3f(-0.9,  0.9,  0.03);
                                        glColor4f(0.2,0.2,0.2,0.0);
                                        glVertex3f(-0.9,  0.9, -0.03);
                                    glEnd();
                                    glEnable (GL_TEXTURE_2D);
                                glDepthMask(GL_TRUE);
                                glDisable(GL_CULL_FACE);
                            glDisable(GL_BLEND);

                            glPopMatrix ();

                        	}
							iArchInd++;
						}
                    }else{
                    /**Current play**/

                      /*  glPushMatrix ();
                        glEnable (GL_DEPTH_TEST);
                        glEnable(GL_BLEND);
                            glColor3f (255.0, 255.0, 255.0);

                            glTranslatef ((iTemp-RESERVED_OBJECT)*(0.3*OglScene->__fArchiveSize__),
                                          -4,
                                          0.0);
                            glRotated(230,0,10,0);
                            glColor4f(  1,
                                        1,
                                        1,0.0);
                            glBindTexture (GL_TEXTURE_2D, OglObject[iTemp].hTexture);
                            glScalef(   OglScene->__fArchiveSize__,
                                        OglScene->__fArchiveSize__,
                                        OglScene->__fArchiveSize__) ;
                            glCallList (4);
                        glPopMatrix ();*/

                    }



                }


}

/**PLAYER*************************************************************************************/
/**Start**/

if (OglScene->__iHideToolbar__==0)
{

                 glEnable(GL_BLEND);
                 glEnable (GL_DEPTH_TEST);


                glPushMatrix ();


                glTranslatef (OglObject[3].x, OglObject[3].y,OglObject[3].z);
                glBindTexture (GL_TEXTURE_2D, j3dtb->Play);

                if (mode == GL_SELECT)
                {glLoadName(6);}
                /*if (OglScene->__LastClicked__==2)
                {	glDisable (GL_TEXTURE_2D);

                    glColor3f(fMyColor,0.5,1.0);
                    glPolygonMode( GL_FRONT_AND_BACK, GL_LINE );
                    glLineWidth( 3.0f );
                }*/
                glColor4f(1.0,1.0,1.0,1.0);

                /*if (iFXState1==1)
                {myFX(0,mode);}*/

                glScalef(0.4F,0.4F,0.4F) ;
                glCallList (3);
                /*if (OglScene->__LastClicked__==2)
                {			glEnable (GL_TEXTURE_2D);
                    glColor3f(1.0,1.0,1.0);
                    glPolygonMode( GL_FRONT_AND_BACK, GL_FILL);}*/

                glPopMatrix ();


/**Stop**/
                glPushMatrix ();
                glTranslatef (OglObject[6].x, OglObject[6].y,OglObject[6].z);
                glBindTexture (GL_TEXTURE_2D, j3dtb->Stop);

                if (mode == GL_SELECT)
                {glLoadName(7);}
                /*if (OglScene->__LastClicked__==7)
                {	glDisable (GL_TEXTURE_2D);

                    glColor3f(fMyColor,0.5,1.0);
                    glPolygonMode( GL_FRONT_AND_BACK, GL_LINE );
                    glLineWidth( 3.0f );
                }*/


                glColor4f(1.0,1.0,1.0,1.0);
                 glScalef(0.4F,0.4F,0.4F) ;
                glCallList (3);


                /*if (OglScene->__LastClicked__==7)
                {			glEnable (GL_TEXTURE_2D);
                    glColor3f(1.0,1.0,1.0);
                    glPolygonMode( GL_FRONT_AND_BACK, GL_FILL);}*/
                glPopMatrix ();




                /**Next**/
                glPushMatrix ();
                glTranslatef (OglObject[5].x, OglObject[5].y,OglObject[5].z);
                glBindTexture (GL_TEXTURE_2D, j3dtb->Next);

                if (mode == GL_SELECT)
                {glLoadName(8);}
                /*if (OglScene->__LastClicked__==8)
                {	glDisable (GL_TEXTURE_2D);

                    glColor3f(fMyColor,0.5,1.0);
                    glPolygonMode( GL_FRONT_AND_BACK, GL_LINE );
                    glLineWidth( 3.0f );
                }*/
                glScalef(0.4F,0.4F,0.4F) ;

                glColor4f(1.0,1.0,1.0,1.0);
                /*if (iFXState1==4)
                {myFX(0,mode);}*/
                glCallList (3);


                /*if (OglScene->__LastClicked__==8)
                {			glEnable (GL_TEXTURE_2D);
                    glColor3f(1.0,1.0,1.0);
                    glPolygonMode( GL_FRONT_AND_BACK, GL_FILL);}*/
                glPopMatrix ();


                /**Previous**/
                glPushMatrix ();
                glTranslatef (OglObject[4].x, OglObject[4].y,OglObject[4].z);
                glBindTexture (GL_TEXTURE_2D, j3dtb->Prev);

                if (mode == GL_SELECT)
                {glLoadName(9);}
                glScalef(0.4F,0.4F,0.4F) ;

                glColor3f(1.0,1.0,1.0);
                glCallList (3);


                /*if (OglScene->__LastClicked__==9)
                {			glEnable (GL_TEXTURE_2D);
                    glColor3f(1.0,1.0,1.0);
                    glPolygonMode( GL_FRONT_AND_BACK, GL_FILL);}*/
                glPopMatrix ();

 /**TOOLBAR FX**/
                if ( OglObject[3].fx == 1)
                {
                    glPushMatrix ();

                    glTranslatef (OglObject[3].x, OglObject[3].y,OglObject[3].z);
                    glBindTexture (GL_TEXTURE_2D, j3dtb->Play);
                    #ifdef DEBUG_OGLPAN
                        /*g_print("LOG-OglPanel.c::draw_object: OglObject[3].fx -> %i ToolbarFx[3].fFxValue1 -> %f\n",OglObject[3].fx,ToolbarFx[3].fFxValue1);*/
                         g_print("LOG-OglPanel.c::draw_object: ToolbarFx[3].fFxValue2 -> %f\n",ToolbarFx[3].fFxValue2);
                    #endif
                    myFX(2,mode,&ToolbarFx[3].fFxValue1,NULL,3.0);
                    myFX(1,mode,&ToolbarFx[3].fFxValue2,&ToolbarFx[3].iFxState,0.0);
                    glScalef(ToolbarFx[3].fFxValue1,ToolbarFx[3].fFxValue1,ToolbarFx[3].fFxValue1) ;
                    glColor4f(1.0,1.0,1.0,ToolbarFx[3].fFxValue2);
                    glCallList (7);
                    if(ToolbarFx[3].iFxState==0)
                    {OglObject[3].fx = 0;}
                    glPopMatrix ();
                }
                if ( OglObject[6].fx == 1)
                {
                    glPushMatrix ();

                    glTranslatef (OglObject[6].x, OglObject[6].y,OglObject[6].z);
                    glBindTexture (GL_TEXTURE_2D, j3dtb->Stop);
                    #ifdef DEBUG_OGLPAN
                        g_print("LOG-OglPanel.c::draw_object: OglObject[6].fx -> %i ToolbarFx[6].fFxValue1 -> %f\n",OglObject[6].fx,ToolbarFx[6].fFxValue1);
                    #endif
                    myFX(2,mode,&ToolbarFx[6].fFxValue1,NULL,1.5);
                    myFX(1,mode,&ToolbarFx[6].fFxValue2,&ToolbarFx[6].iFxState,0.0);
                    glScalef(ToolbarFx[6].fFxValue1,ToolbarFx[6].fFxValue1,ToolbarFx[6].fFxValue1) ;
                    glColor4f(1.0,1.0,1.0,ToolbarFx[6].fFxValue2);
                    glCallList (7);
                    if(ToolbarFx[6].iFxState==0)
                    {OglObject[6].fx = 0;}
                    glPopMatrix ();
                }
                if ( OglObject[5].fx == 1)
                {
                    glPushMatrix ();

                    glTranslatef (OglObject[5].x, OglObject[5].y,OglObject[5].z);
                    glBindTexture (GL_TEXTURE_2D, j3dtb->Next);
                    #ifdef DEBUG_OGLPAN
                        g_print("LOG-OglPanel.c::draw_object: OglObject[6].fx -> %i ToolbarFx[6].fFxValue1 -> %f\n",OglObject[6].fx,ToolbarFx[6].fFxValue1);
                    #endif
                    myFX(2,mode,&ToolbarFx[5].fFxValue1,NULL,1.5);
                    myFX(1,mode,&ToolbarFx[5].fFxValue2,&ToolbarFx[5].iFxState,0.0);
                    glScalef(ToolbarFx[5].fFxValue1,ToolbarFx[5].fFxValue1,ToolbarFx[5].fFxValue1) ;
                    glColor4f(1.0,1.0,1.0,ToolbarFx[5].fFxValue2);
                    glCallList (7);
                    if(ToolbarFx[5].iFxState==0)
                    {OglObject[5].fx = 0;}
                    glPopMatrix ();
                }
                if ( OglObject[4].fx == 1)
                {
                    glPushMatrix ();

                    glTranslatef (OglObject[4].x, OglObject[4].y,OglObject[4].z);
                    glBindTexture (GL_TEXTURE_2D, j3dtb->Prev);
                    #ifdef DEBUG_OGLPAN
                        g_print("LOG-OglPanel.c::draw_object: OglObject[6].fx -> %i ToolbarFx[6].fFxValue1 -> %f\n",OglObject[6].fx,ToolbarFx[6].fFxValue1);
                    #endif
                    myFX(2,mode,&ToolbarFx[4].fFxValue1,NULL,2.0);
                    myFX(1,mode,&ToolbarFx[4].fFxValue2,&ToolbarFx[4].iFxState,0.0);
                    glScalef(ToolbarFx[4].fFxValue1,ToolbarFx[4].fFxValue1,ToolbarFx[4].fFxValue1) ;
                    glColor4f(1.0,1.0,1.0,ToolbarFx[4].fFxValue2);
                    glCallList (7);
                    if(ToolbarFx[4].iFxState==0)
                    {OglObject[4].fx = 0;}
                    glPopMatrix ();
                }
}





/*if (OglScene->__AlivPart__==1)
{
	glPushMatrix ();
	glEnable (GL_BLEND);
	glDisable (GL_DEPTH_TEST);
	glBlendFunc (GL_SRC_ALPHA, GL_ONE);
	DrawSceneParticule(mode); //mode is for select ( opengl pick ) or rendering
	glPopMatrix ();
}*/
glPopAttrib();
   /* glDisable(GL_LIGHTING);
    glDisable(GL_LIGHT0);
    glDisable(GL_FOG);*/
	/**************************************************************************/
	/*************************Enter 2D MODE************************************/
	glPushMatrix ();

	SDL_GL_Enter2DMode(widget);
	/************************Sponsors System***********************************/

	/*if (OglScene->__LastClicked__==10)
	{
		glDisable (GL_TEXTURE_2D);
		glColor3f(fMyColor,0.5,1.0);
		glPolygonMode( GL_FRONT_AND_BACK, GL_LINE );
		glLineWidth( 3.0f );
	}*/
	/*glCallList (2);*/

	/*if (OglScene->__LastClicked__==10)
	{
		glEnable (GL_TEXTURE_2D);
		glColor3f(1.0,1.0,1.0);
		glPolygonMode( GL_FRONT_AND_BACK, GL_FILL);
	}*/
	if (OglScene->__ActSpons__==1)
	{
        if (mode == GL_SELECT)
        {glLoadName(1);}


        if (OglScene->__iInitTextEffect__!=0)
            {
            myFX(0,mode,&OglScene->fTextFade,&OglScene->__iInitTextEffect__,1.0);
            #ifdef DEBUG_ODEBUG_OGLPAN
               /* g_print("Invocking FX Engine\n");
                g_print("fTextFade %f\n",fTextFade);*/
            #endif
            }else{

                if (OglScene->__iAlwaysDispInf__==0)
                {
                    if (OglScene->fTextFade>0.0)
                    {
                        myFX(1,mode,&OglScene->fTextFade,NULL,0.0);
                    }else{
                        OglScene->fTextFade=0.0;}
                }else{
                        OglScene->fTextFade=1.0;}
            }
		DrawSceneText(OglScene->fTextFade);
	}
	SDL_GL_Leave2DMode(widget);

           /* glBindTexture(GL_TEXTURE_2D, int(__logo__[6]))
            glScalef (float(__logo__[7])/float(10), float(__logo__[8])/float(10), 1.0)*/

    glPopAttrib();
    iTemp=0;
	return;
}


void myFX(int FX,GLenum mode,float *fValue,int *iStatus,float fStopFx)
{
    /** FX Function
    *** 0 Mean fadein
    *** 1 Mean fadeout

    **/
    if (mode != GL_SELECT)
      {
            switch(FX)
            {
                /**Fade in**/
                case 0:
                        if (*fValue<fStopFx)
                        {
                            *fValue=*fValue+0.005;
                        }else{
                            *iStatus=0;
                            }
                        #ifdef DEBUG_OGLPAN
                          /*  g_print("FX Case 0 Enabled\n");*/
                        #endif
                    break;
                /**Fade out**/
                case 1:
                        if (*fValue>fStopFx)
                        {
                            #ifdef DEBUG_OGLPAN
                                g_print("FX Fade out enabled\n");
                            #endif
                            *fValue=*fValue-0.010;
                            if (iStatus!=NULL)
                            {*iStatus=1;}
                        }else{
                            #ifdef DEBUG_OGLPAN
                                g_print("FX Fade out finish\n");
                            #endif
                            if (iStatus!=NULL)
                            {*iStatus=0;}
                            }
                        #ifdef DEBUG_OGLPAN
                           /* g_print("FX Case 1 Enabled\n");*/
                        #endif
                        break;
                /** Zoom in**/
                case 2:
                        if (*fValue<fStopFx)
                        {
                            #ifdef DEBUG_OGLPAN
                                g_print("FX Case 2 Enabled\n");
                            #endif
                            *fValue=*fValue+0.01;
                            if (iStatus!=NULL)
                            {*iStatus=1;}
                        }else{
                            #ifdef DEBUG_OGLPAN
                                g_print("FX Case 2 Disabled\n");
                            #endif
                            if (iStatus!=NULL)
                            {*iStatus=0;}
                            *fValue=0;
                            }

                    break;
                /** Zoom out**/
                case 3:


                    break;
                default:
                    return;
            }

      }


             /*     if (OglScene->__iInitTextEffect__==1)
			{
                    fTextFade=0.0;
                    OglScene->__iInitTextEffect__=0;
			}
            if (fTextFade < 1.0)
            {
				fTextFade=fTextFade+fFadingStep;
            }*/
  return;
}


/**Receive how to rotate current matrix**/
void SceneStyleAnim(float *fAngleObject,int iRot)
{
    /*g_print("Angle modifié %f\n",*fAngleObject);*/
    switch(iRot)
    {
        case 0:
            if (*fAngleObject<360.0)
            {
                *fAngleObject=*fAngleObject+OglScene->__speed__;
                glRotatef(*fAngleObject,0.0,10.0,0.0);
            }else{
                *fAngleObject=0.0;
            }
        break;
        case 1:
            if (*fAngleObject>-360.0)
            {
                *fAngleObject=*fAngleObject-OglScene->__speed__;
                glRotatef(*fAngleObject,0.0,10.0,0.0);
            }else{
                *fAngleObject=0.0;
            }
        break;
        case 2:
            if (*fAngleObject<360.0)
            {
                *fAngleObject=*fAngleObject+OglScene->__speed__;
                glRotatef(*fAngleObject, 1.0f, 0.0f, 0.0f);
            }else{
                *fAngleObject=0.0;
            }
        break;
        case 3:
            if (*fAngleObject>-360.0)
            {
                *fAngleObject=*fAngleObject-OglScene->__speed__;
                glRotatef(*fAngleObject, 1.0f, 0.0f, 0.0f);
            }else{
                *fAngleObject=0.0;
            }
        break;
    }
return;
}


void DrawSceneText(float myAlpha)
{

			#ifdef DEBUG_OGLPAN
				/*printf("OglPanel.c::DrawSceneText: Alpha -> %f \n", (float)myAlpha);*/
			#endif
			if (OglScene->iNbrEntrySpon>0)
			{
				if (DataTxtEntry[OglScene->iNbrEntrySpon].type==0)
				{
					int i;
					glColor4f (DataTxtEntry[OglScene->ptTexte].red, DataTxtEntry[OglScene->ptTexte].green, DataTxtEntry[OglScene->ptTexte].blue, myAlpha);

					for(i=0 ; i<5 ; i++) // pour chaque message ( 1 a 5 )
					{
						glBindTexture(GL_TEXTURE_2D, DataTxtEntry[OglScene->ptTexte].cMsg[i].cTexture);
						glBegin(GL_TRIANGLE_STRIP);

						glTexCoord2f(DataTxtEntry[OglScene->ptTexte].cMsg[i].cTexcoord[0], DataTxtEntry[OglScene->ptTexte].cMsg[i].cTexcoord[1]);
						glVertex2i(DataTxtEntry[OglScene->ptTexte].cMsg[i].cX,
						           DataTxtEntry[OglScene->ptTexte].cMsg[i].cY
						           );/* X , Y */

						glTexCoord2f(DataTxtEntry[OglScene->ptTexte].cMsg[i].cTexcoord[2], DataTxtEntry[OglScene->ptTexte].cMsg[i].cTexcoord[1]);
						glVertex2i(DataTxtEntry[OglScene->ptTexte].cMsg[i].cX+DataTxtEntry[OglScene->ptTexte].cMsg[i].cW,
						           DataTxtEntry[OglScene->ptTexte].cMsg[i].cY
						           );/*X+W , Y */

						glTexCoord2f(DataTxtEntry[OglScene->ptTexte].cMsg[i].cTexcoord[0], DataTxtEntry[OglScene->ptTexte].cMsg[i].cTexcoord[3]);
						glVertex2i(DataTxtEntry[OglScene->ptTexte].cMsg[i].cX,
						           DataTxtEntry[OglScene->ptTexte].cMsg[i].cY +DataTxtEntry[OglScene->ptTexte].cMsg[i].cH
						           );/* X ,Y+H*/

						glTexCoord2f(DataTxtEntry[OglScene->ptTexte].cMsg[i].cTexcoord[2], DataTxtEntry[OglScene->ptTexte].cMsg[i].cTexcoord[3]);
						glVertex2i(DataTxtEntry[OglScene->ptTexte].cMsg[i].cX+DataTxtEntry[OglScene->ptTexte].cMsg[i].cW,
						           DataTxtEntry[OglScene->ptTexte].cMsg[i].cY+DataTxtEntry[OglScene->ptTexte].cMsg[i].cH
						           );/*X+W,Y+H*/
						glEnd();
					}
				}
			}
}
